@@ -4,8 +4,6 @@
 #define __h_message
 
 #include "common.h"
-#include "lber.h"
-#include "ldap.h"
 
 extern PyObject *LDAPmessage_to_python(LDAP *ld, LDAPMessage *m, int add_ctrls,
                                        int add_intermediates);
