@@ -5,6 +5,7 @@
 #include "LDAPObject.h"
 #include "ldapcontrol.h"
 #include "options.h"
+#include "berval.h"
 
 void
 set_timeval_from_double(struct timeval *tv, double d)
@@ -40,6 +41,7 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 {
     int res;
     int intval;
+    unsigned int uintval;
     double doubleval;
     char *strval;
     struct timeval tv;
@@ -56,8 +58,12 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
     switch (option) {
     case LDAP_OPT_API_INFO:
     case LDAP_OPT_API_FEATURE_INFO:
+    case LDAP_OPT_DESC:
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SSF:
+#endif
+#ifdef LDAP_OPT_X_TLS_PEERCERT
+    case LDAP_OPT_X_TLS_PEERCERT:
 #endif
         /* Read-only options */
         PyErr_SetString(PyExc_ValueError, "read-only option");
@@ -92,6 +98,9 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 #ifdef LDAP_OPT_X_TLS_PROTOCOL_MIN
     case LDAP_OPT_X_TLS_PROTOCOL_MIN:
 #endif
+#ifdef LDAP_OPT_X_TLS_PROTOCOL_MAX
+    case LDAP_OPT_X_TLS_PROTOCOL_MAX:
+#endif
 #ifdef LDAP_OPT_X_TLS_REQUIRE_SAN
     case LDAP_OPT_X_TLS_REQUIRE_SAN:
 #endif
@@ -112,10 +121,19 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
         ptr = &intval;
         break;
 
+#ifdef LDAP_OPT_TCP_USER_TIMEOUT
+    case LDAP_OPT_TCP_USER_TIMEOUT:
+#endif
+        if (!PyArg_Parse(value, "I:set_option", &uintval))
+            return 0;
+        ptr = &uintval;
+        break;
+
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SSF_MIN:
     case LDAP_OPT_X_SASL_SSF_MAX:
     case LDAP_OPT_X_SASL_SSF_EXTERNAL:
+    case LDAP_OPT_X_SASL_MAXBUFSIZE:
         if (!PyArg_Parse(value, "k:set_option", &blen))
             return 0;
         ptr = &blen;
@@ -140,9 +158,15 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 #ifdef LDAP_OPT_X_TLS_CRLFILE
     case LDAP_OPT_X_TLS_CRLFILE:
 #endif
+#ifdef LDAP_OPT_X_TLS_ECNAME
+    case LDAP_OPT_X_TLS_ECNAME:
+#endif
 #endif
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SECPROPS:
+#endif
+#ifdef LDAP_OPT_SOCKET_BIND_ADDRESSES
+    case LDAP_OPT_SOCKET_BIND_ADDRESSES:
 #endif
         /* String valued options */
         if (!PyArg_Parse(value, "s:set_option", &strval))
@@ -183,8 +207,8 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
         }
         else {
             PyErr_Format(PyExc_ValueError,
-                         "timeout must be >= 0 or -1/None for infinity, got %d",
-                         option);
+                         "timeout must be >= 0 or -1/None for infinity, got %f",
+                         doubleval);
             return 0;
         }
         break;
@@ -250,10 +274,12 @@ LDAP_get_option(LDAPObject *self, int option)
 {
     int res;
     int intval;
+    unsigned int uintval;
     struct timeval *tv;
     LDAPAPIInfo apiinfo;
     LDAPControl **lcs;
     char *strval;
+    struct berval berbytes;
 #if HAVE_SASL
     /* unsigned long */
     ber_len_t blen;
@@ -263,6 +289,7 @@ LDAP_get_option(LDAPObject *self, int option)
 
     switch (option) {
 #ifdef HAVE_SASL
+    case LDAP_OPT_X_SASL_SECPROPS:
     case LDAP_OPT_X_SASL_SSF_EXTERNAL:
         /* Write-only options */
         PyErr_SetString(PyExc_ValueError, "write-only option");
@@ -320,6 +347,9 @@ LDAP_get_option(LDAPObject *self, int option)
 #ifdef LDAP_OPT_X_TLS_PROTOCOL_MIN
     case LDAP_OPT_X_TLS_PROTOCOL_MIN:
 #endif
+#ifdef LDAP_OPT_X_TLS_PROTOCOL_MAX
+    case LDAP_OPT_X_TLS_PROTOCOL_MAX:
+#endif
 #ifdef LDAP_OPT_X_TLS_REQUIRE_SAN
     case LDAP_OPT_X_TLS_REQUIRE_SAN:
 #endif
@@ -345,10 +375,20 @@ LDAP_get_option(LDAPObject *self, int option)
             return option_error(res, "ldap_get_option");
         return PyInt_FromLong(intval);
 
+#ifdef LDAP_OPT_TCP_USER_TIMEOUT
+    case LDAP_OPT_TCP_USER_TIMEOUT:
+#endif
+        /* unsigned int options */
+        res = LDAP_int_get_option(self, option, &uintval);
+        if (res != LDAP_OPT_SUCCESS)
+            return option_error(res, "ldap_get_option");
+        return PyLong_FromUnsignedLong(uintval);
+
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SSF:
     case LDAP_OPT_X_SASL_SSF_MIN:
     case LDAP_OPT_X_SASL_SSF_MAX:
+    case LDAP_OPT_X_SASL_MAXBUFSIZE:
         /* ber_len_t options (unsigned long)*/
         res = LDAP_int_get_option(self, option, &blen);
         if (res != LDAP_OPT_SUCCESS)
@@ -383,9 +423,11 @@ LDAP_get_option(LDAPObject *self, int option)
 #ifdef LDAP_OPT_X_TLS_PACKAGE
     case LDAP_OPT_X_TLS_PACKAGE:
 #endif
+#ifdef LDAP_OPT_X_TLS_ECNAME
+    case LDAP_OPT_X_TLS_ECNAME:
+#endif
 #endif
 #ifdef HAVE_SASL
-    case LDAP_OPT_X_SASL_SECPROPS:
     case LDAP_OPT_X_SASL_MECH:
     case LDAP_OPT_X_SASL_REALM:
     case LDAP_OPT_X_SASL_AUTHCID:
@@ -393,6 +435,9 @@ LDAP_get_option(LDAPObject *self, int option)
 #ifdef LDAP_OPT_X_SASL_USERNAME
     case LDAP_OPT_X_SASL_USERNAME:
 #endif
+#endif
+#ifdef LDAP_OPT_SOCKET_BIND_ADDRESSES
+    case LDAP_OPT_SOCKET_BIND_ADDRESSES:
 #endif
         /* String-valued options */
         res = LDAP_int_get_option(self, option, &strval);
@@ -406,6 +451,19 @@ LDAP_get_option(LDAPObject *self, int option)
         ldap_memfree(strval);
         return v;
 
+#ifdef HAVE_TLS
+#ifdef LDAP_OPT_X_TLS_PEERCERT
+    case LDAP_OPT_X_TLS_PEERCERT:
+#endif
+#endif
+        /* Options dealing with raw data */
+        res = LDAP_int_get_option(self, option, &berbytes);
+        if (res != LDAP_OPT_SUCCESS)
+            return option_error(res, "ldap_get_option");
+        v = LDAPberval_to_object(&berbytes);
+        ldap_memfree(berbytes.bv_val);
+        return v;
+
     case LDAP_OPT_TIMEOUT:
     case LDAP_OPT_NETWORK_TIMEOUT:
         /* Double-valued timeval options */
